package org.jeecg.modules.bytedance.advertise.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.modules.bytedance.advertise.dockapi.MarketingService;
import org.jeecg.modules.bytedance.advertise.entity.ByteDanceAdvertisePlan;
import org.jeecg.modules.bytedance.advertise.entity.ByteDanceCreative;
import org.jeecg.modules.bytedance.advertise.entity.ByteDanceVideoInfo;
import org.jeecg.modules.bytedance.advertise.entity.BytedanceImageInfo;
import org.jeecg.modules.bytedance.advertise.mapper.ByteDanceCreativeMapper;
import org.jeecg.modules.bytedance.advertise.mapper.MonitorBytedanceActionTrackUrlMapper;
import org.jeecg.modules.bytedance.advertise.service.IByteDanceAdvertisePlanService;
import org.jeecg.modules.bytedance.advertise.service.IByteDanceAdvertiserDataService;
import org.jeecg.modules.bytedance.advertise.service.IByteDanceCreativeMaterialService;
import org.jeecg.modules.bytedance.advertise.service.IByteDanceVideoInfoService;
import org.jeecg.modules.bytedance.advertise.service.IBytedanceImageInfoService;
import org.jeecg.modules.bytedance.common.entity.CtopOauthToken;
import org.jeecg.modules.bytedance.common.entity.UserAllocation;
import org.jeecg.modules.bytedance.common.service.ICtopOauthTokenService;
import org.jeecg.modules.bytedance.common.service.IUserAllocationService;
import org.jeecg.modules.bytedance.common.utils.Check;
import org.jeecg.modules.bytedance.common.utils.CorpWexinUtils;
import org.jeecg.modules.bytedance.common.utils.HttpUtils;
import org.jeecg.modules.bytedance.common.utils.PropertiesUtils;
import org.jeecg.modules.bytedance.common.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**
 * @author 宋英豪
 */
@Slf4j
@Service
@Primary
public class ByteDanceAdvertiserDataServiceImpl implements IByteDanceAdvertiserDataService {
    @Value("${bytedance.appId}")
    private String bytedanceAppId;
    @Value("${bytedance.secret}")
    private String bytedanceSecret;
    @Value("${bytedance.url.api-prefix}")
    private String bytedanceApiUrl;
    @Value("${bytedance.url.ad-get}")
    private String bytedanceAdGetUrl;

    @Autowired
    private ICtopOauthTokenService tokenService;
    @Autowired
    private IByteDanceAdvertisePlanService advertisePlanService;
    @Autowired
    private IByteDanceCreativeMaterialService creativeMaterialService;
    @Autowired
    private ByteDanceCreativeMapper creativeMapper;
    @Autowired
    private IByteDanceVideoInfoService videoInfoService;
    @Autowired
    private IUserAllocationService userAllocationService;
    @Resource
    private CorpWexinUtils corpWexinUtils;
    @Resource
    private MonitorBytedanceActionTrackUrlMapper monitorBytedanceActionTrackUrlMapper;
    @Resource
    private MarketingService marketingServiceImpl;


    @Override
    public JSONObject getAdvertiserList(String token, Long accountId) {
        JSONObject returnJson = new JSONObject();
        try {
            String url = "https://ad.oceanengine.com/open_api/2/majordomo/advertiser/select/";
            Map<String, String> headers = new HashMap<>();
            headers.put("Content-Type", "application/json");
            headers.put("Access-Token", token);
            TreeMap<String, Object> params = new TreeMap<>();
            params.put("advertiser_id", accountId);
            String result = HttpUtils.httpGetRequest(url, headers, params);
            JSONObject jsonObject = JSONObject.parseObject(result);
            System.err.println(jsonObject);
            if (Check.isNull(jsonObject)) {
                throw new Exception("返回结果为空");
            }

            int code = jsonObject.getInteger("code");
            if (code == 0) {
                JSONObject data = jsonObject.getJSONObject("data");
                if (Check.isNull(data)) {
                    throw new Exception("data为空");
                }
                JSONArray list = data.getJSONArray("list");
                if (Check.isNull(list)) {
                    throw new Exception("账户列表为空");

                }
                returnJson.put("code", 0);
                returnJson.put("data", list);

            } else {
                returnJson.put("code", -1);
            }


        } catch (Exception e) {
            e.printStackTrace();
            returnJson.put("code", -1);

        }


        return returnJson;


    }

    @Override
    public Map<String, Object> getAdvertiserInfo(String accountId, String token) {
        Map<String, Object> resultMap = new HashMap<>();
        String url = bytedanceApiUrl + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_advertiser_info");
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", token);
        TreeMap<String, Object> params = new TreeMap<>();
        params.put("advertiser_ids", "[" + accountId + "]");
        String result = HttpUtils.httpGetRequest(url, headers, params);
        JSONObject jsonObject = JSONObject.parseObject(result);
        int code = jsonObject.getInteger("code");

        if (code != 0) {
            log.info("获取广告主信息接口异常==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            resultMap.put("success", false);
            resultMap.put("code", -1);
            resultMap.put("message", "获取广告主信息接口异常");
            return resultMap;
        }
        JSONArray data = jsonObject.getJSONArray("data");
        if (null == data || data.size() <= 0) {
            resultMap.put("success", false);
            resultMap.put("code", -1);
            resultMap.put("message", "广告主信息不存在");
            return resultMap;
        }
        for (int i = 0; i < data.size(); i++) {
            JSONObject dataObject = data.getJSONObject(i);
            resultMap.put("name", dataObject.getString("name"));
        }
        resultMap.put("success", true);
        resultMap.put("code", 0);
        resultMap.put("message", "获取广告主信息成功");
        return resultMap;
    }


    @Override
    public JSONArray advertiserList(String token) {
        String url = "https://ad.oceanengine.com/open_api/oauth2/advertiser/get/";
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        TreeMap<String, Object> params = new TreeMap<>();
        params.put("access_token", token);
        params.put("secret", bytedanceSecret);
        params.put("app_id", bytedanceAppId);
        String result = HttpUtils.httpGetRequest(url, headers, params);
        JSONObject jsonObject = JSONObject.parseObject(result);
        log.info("返回信息：{}", jsonObject);
        Integer code = jsonObject.getInteger("code");
        if (code == 0) {
            JSONObject dataJson = jsonObject.getJSONObject("data");
            if (!Check.isNull(dataJson)) {
                JSONArray listArr = dataJson.getJSONArray("list");
                if (!Check.isNull(listArr)) {
                    return listArr;
                }
            }
        }
        return new JSONArray();
    }

    @Override
    public ByteDanceAdvertisePlan getSinglePlanById(CtopOauthToken token, String id) {
        JSONArray getIds = new JSONArray();
        getIds.add(id);
        // 请求地址
        String url = bytedanceApiUrl + bytedanceAdGetUrl;
        // 请求参数
        Map<String, Object> filtering = new HashMap<>();
        filtering.put("ids", getIds);
        JSONObject param = new JSONObject();
        param.put("advertiser_id", token.getAccountId());
        param.put("page", 1);
        param.put("page_size", 100);
        param.put("filtering", filtering);
        JSONObject resultObject = HttpUtils.bytedanceGetRequest(token.getAccessToken(), url, param);
        Integer code = resultObject.getInteger("code");
        if (null == code || !code.equals(0)) {
            log.error("获取广告计划信息接口异常==》accountId:{},message:{}", token.getAccountId(), resultObject.getString("message"));
            return null;
        }
        JSONArray data = resultObject.getJSONObject("data").getJSONArray("list");
        if (null == data || data.isEmpty()) {
            return null;
        }
        JSONObject dataObject = data.getJSONObject(0);
        ByteDanceAdvertisePlan advertisePlan = new ByteDanceAdvertisePlan(dataObject, String.valueOf(token.getAccountId()));
        BigDecimal deepCpabid = dataObject.getBigDecimal("deep_cpabid");
        if (null != deepCpabid) {
            advertisePlan.setDeepCpaBid(deepCpabid);
        }
        return advertisePlan;
    }


    public void getAd(CtopOauthToken token, int pageNum, String ids, String date, String updateDate) {
        JSONArray getIds = null;
        if (null != ids && !"".equals(ids)) {
            String[] idString = ids.split(StringUtils.COMMA);
            if (idString.length > 0) {
                for (int i = 0; i < idString.length; i++) {
                    Long getId = Long.parseLong(idString[i]);
                    getIds.add(getId);
                }
            }
        }

        // 请求地址
        String url = bytedanceApiUrl + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_ad_get");
        // 请求参数
        Map<String, Object> filtering = new HashMap<>();
        if (null != getIds && getIds.size() > 0) {
            filtering.put("ids", getIds);
        }
        if (null != date && !"".equals(date)) {
            filtering.put("ad_create_time", date);
        }
        if (null != updateDate && !"".equals(updateDate)) {
            filtering.put("ad_modify_time", updateDate);
        }

        JSONObject param = new JSONObject();
        param.put("advertiser_id", token.getAccountId());
        param.put("page", pageNum);
        param.put("page_size", 100);
        param.put("filtering", filtering);
        JSONObject resultObject = HttpUtils.bytedanceGetRequest(token.getAccessToken(), url, param);
        Integer code = resultObject.getInteger("code");
        if (null == code || !code.equals(0)) {
            log.error("获取广告计划信息接口异常==》accountId:{},message:{}", token.getAccountId(), resultObject.getString("message"));
            return;
        }
        JSONArray data = resultObject.getJSONObject("data").getJSONArray("list");
        if (null == data || data.isEmpty()) {
            return;
        }
        for (int i = 0; i < data.size(); i++) {
            JSONObject dataObject = data.getJSONObject(i);
            checkBidType(token.getAccountId(), dataObject);
            ByteDanceAdvertisePlan advertisePlan = new ByteDanceAdvertisePlan(dataObject, String.valueOf(token.getAccountId()));
            BigDecimal deepCpabid = dataObject.getBigDecimal("deep_cpabid");
            if (null != deepCpabid) {
                advertisePlan.setDeepCpaBid(deepCpabid);
            }
            advertisePlanService.saveOrUpdate(advertisePlan);
        }
        getAd(token, pageNum + 1, ids, date, updateDate);
    }

    /**
     * 校验转化出价，并预警
     */
    public void checkBidType(Long accountId, JSONObject detail) {
        JSONObject json = userAllocationService.getProjectByAccountId(accountId);
        if (Check.isNull(json) || Check.isNull(detail)) {
            return;
        }
        //转化目标map
        Map<String, String> ocpxMap = new HashMap<>();
        String ocpxTypeStr = json.getString("ocpxActionType");
        if (!Check.isNull(ocpxTypeStr)) {
            JSONArray ocpxArr = JSONArray.parseArray(ocpxTypeStr);
            for (int i = 0; i < ocpxArr.size(); i++) {
                JSONObject obj = ocpxArr.getJSONObject(i);
                ocpxMap.put(obj.getString("ocpxCode"), obj.getString("maxValue"));
            }
        }

        //深度转化目标map
        Map<String, String> deepMap = new HashMap<>();
        String deepTypeStr = json.getString("deepConversionType");
        if (!Check.isNull(deepTypeStr)) {
            JSONArray deepArr = JSONArray.parseArray(deepTypeStr);
            for (int i = 0; i < deepArr.size(); i++) {
                JSONObject obj = deepArr.getJSONObject(i);
                deepMap.put(obj.getString("deepCode"), obj.getString("deepValue"));
            }
        }

        //转化目标
        String ocpxActionType = detail.getString("external_action");
        //转化目标出价
        String cpaBid = detail.getString("cpa_bid");
        //深度转化目标
        String deepConversionType = detail.getString("deep_external_action");
        //深度转化目标出价
        String deepConversionBid = detail.getString("deep_cpabid");

        //计划ID
        Long planId = detail.getLong("ad_id");
        //计划名称
        String planName = detail.getString("name");
        //投放状态
        String status = detail.getString("status");
        //操作状态
        String optStatus = detail.getString("opt_status");
        if ("AD_STATUS_ENABLE".equals(optStatus) && !"AD_STATUS_DISABLE".equals(status)) { // 操作状态为‘启用’，广告计划投放状态 非‘暂停’
            Map<String, String> textMap = new HashMap<>();
            List<JSONObject> lists = creativeMapper.getBytedanceOcpxTypeList();
            for (JSONObject da : lists) {
                textMap.put(da.getString("ocpxActionType"), da.getString("ocpxActionName"));
            }
            String msgType = "";
            String msgBid = "";

            //转化目标标识
            Boolean typeFalg = true;
            //出价标识
            Boolean bidFalg = true;
            //转化目标
            if (!Check.isNull(ocpxActionType)) {
                if (ocpxMap.isEmpty()) {
                    typeFalg = false;
                    String msg = Check.isNull(textMap.get(ocpxActionType)) ? ocpxActionType : textMap.get(ocpxActionType);
                    msgType = "项目未设转化，但计划转化：" + msg;
                } else {
                    //转化出价
                    String ocpxBid = ocpxMap.get(ocpxActionType);
                    //通过转化目标没有查询到转化出价，判断为转化目标不一致，触发关停
                    if (Check.isNull(ocpxBid)) {
                        typeFalg = false;
                        String msg = Check.isNull(textMap.get(ocpxActionType)) ? ocpxActionType : textMap.get(ocpxActionType);
                        msgType = "计划转化：" + msg;
                    } else {
                        //未查询到转化出价，说明出价不一致，触发出价关停。
                        if (Check.isNull(cpaBid)) {
                            bidFalg = false;
                            msgBid = "计划出价为空";
                        } else {
                            BigDecimal cpa = new BigDecimal(cpaBid).multiply(new BigDecimal("1000"));
                            BigDecimal ocpx = new BigDecimal(ocpxBid);
                            if (cpa.compareTo(ocpx) == 1) {
                                //计划转化出价cpa 大于 项目设置最大转化出价，则关停
                                bidFalg = false;
                                msgBid = "计划出价：" + cpaBid;
                            }
                        }
                    }
                }
            }
            //深度转化目标
            if (typeFalg && bidFalg) {
                if (!Check.isNull(deepConversionType) && deepMap.isEmpty()) {
                    typeFalg = false;
                    String msg = Check.isNull(textMap.get(deepConversionType)) ? deepConversionType : textMap.get(deepConversionType);
                    msgType = "项目未设深度转化，但计划深度转化：" + msg;
                } else if (!Check.isNull(deepConversionType)) {
                    String deepBid = deepMap.get(deepConversionType);
                    if (Check.isNull(deepBid)) {
                        //通过key未获取值，说明不包含该目标，进行关停
                        typeFalg = false;
                        String msg = Check.isNull(textMap.get(deepConversionType)) ? deepConversionType : textMap.get(deepConversionType);
                        msgType = "计划深度转化：" + msg;
                    } else {
                        //未查询到转化出价，说明出价不一致，触发出价关停。
                        if (Check.isNull(deepConversionBid)) {
                            bidFalg = false;
                            msgBid = "计划深度出价为空";
                        } else {
                            BigDecimal cpa = new BigDecimal(deepConversionBid).multiply(new BigDecimal("1000"));
                            BigDecimal ocpx = new BigDecimal(deepBid);
                            if (cpa.compareTo(ocpx) == 1) {
                                //计划转化出价cpa 大于 项目设置最大转化出价，则关停
                                bidFalg = false;
                                msgBid = "计划深度出价：" + deepConversionBid;
                            }
                        }
                    }
                }
            }
            if (!typeFalg) { // 出价方式设置非法
                String text = getBidTypeText(json.getString("projectName"), accountId, planId, planName, msgType);
                sendMsg(accountId, planId, text);
                detail.put("opt_status", "opt_AD_STATUS_DISABLE");
            }
            if (!bidFalg) {
                String text = getOverBidMessage(json.getString("projectName"), accountId, planId, planName, msgBid);
                sendMsg(accountId, planId, text);
                detail.put("opt_status", "opt_AD_STATUS_DISABLE");
            }
        }

    }

    private void sendMsg(Long accountId, Long planId, String text) {
        //关停计划
        CtopOauthToken token = tokenService.getTokenByAccountId(accountId);
        marketingServiceImpl.updPlanStatus(token, Arrays.asList(planId), "disable");
        log.info("出价方式预警，accountId:{}==>计划id:{}=====转化目标不一致,计划已【关停】============", accountId, planId);
        try {
            UserAllocation account = userAllocationService.getByAccountId(accountId);
            String userId = "";
            if (!Check.isNull(account)) {
                userId = account.getUserId();
            }
            JSONObject jsonObject = monitorBytedanceActionTrackUrlMapper.getWChatIdByUserId(userId);
            corpWexinUtils.sendMessageByWeChatId(jsonObject, text);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    //转化方式错误消息生成
    private String getBidTypeText(String projectName, Long advertiserId, Long unitId, String unit_name, String msgType) {
        StringBuilder text = new StringBuilder();
        text.append("出价方式错误预警").append("<br/>")
                .append("您的项目：").append(projectName + "，").append("<br/>")
                .append("下的账户：").append(advertiserId + "，").append("<br/>")
                .append("广告计划id：").append(unitId).append("<br/>")
                .append("广告计划名称：").append(unit_name).append("<br/>")
                .append("出价方式设置错误。").append("<br/>")
                .append(msgType).append("<br/>")
                .append("系统已执行暂停此计划，请您及时查看！");
        return text.toString();
    }

    //出价过高预警消息生成
    private String getOverBidMessage(String projectName, Long accountId, Long planId, String planName, String msgBid) {
        StringBuilder text = new StringBuilder();
        text.append("出价过高预警").append("<br/>")
                .append("您的项目：").append(projectName + "，").append("<br/>")
                .append("下的账户：").append(accountId + "，").append("<br/>")
                .append("广告计划id为：").append(planId).append("<br/>")
                .append("广告计划名称为：").append(planName).append("<br/>")
                .append("出价设置过高。").append("<br/>")
                .append(msgBid).append("<br/>")
                .append("系统已执行暂停此计划，请您及时查看！");
        return text.toString();
    }


    @Override
    public Map<String, Object> getAdvertiserPlan(CtopOauthToken token, String ids, String date, String updateDate) {
        getAd(token, 1, ids, date, updateDate);
        Map<String, Object> resultMap = new HashMap<>();
        resultMap.put("code", 0);
        resultMap.put("message", "获取广告计划信息成功");
        return resultMap;
    }

    @Override
    public Map<String, Object> getAdvertiserBudget(String accountId) {
        Map<String, Object> resultMap = new HashMap<>();
        CtopOauthToken cTopOauthToken = tokenService.getOauthTokenByAccountId(accountId);
        //2: 根据token以及用户id获取用户信息数据
        String url = bytedanceApiUrl + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_campaign_get");
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", cTopOauthToken.getAccessToken());

        TreeMap<String, Object> params = new TreeMap<>();
        params.put("advertiser_ids", "[" + cTopOauthToken.getAccountId() + "]");
        String result = HttpUtils.httpGetRequest(url, headers, params);
        JSONObject jsonObject = JSONObject.parseObject(result);
        Integer code = jsonObject.getInteger("code");

        if (null == code || !code.equals(0)) {
            log.error("获取广告预算信息接口异常==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "获取广告预算信息接口异常");
            return resultMap;
        }
        JSONArray data = jsonObject.getJSONArray("data");
        if (null == data || data.isEmpty()) {
            log.info("广告主预算信息不存在==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            resultMap.put("code", 0);
            resultMap.put("message", "获取广告主预算信息完成");
            return resultMap;
        }
        resultMap.put("code", 0);
        resultMap.put("message", "获取广告预算信息完成");
        return resultMap;
    }


    private void getCreativeByPage(CtopOauthToken token, String date, int pageNum) {
        JSONObject resultObject = getCreative(token, date, pageNum);
        Integer code = resultObject.getInteger("code");
        if (null == code || !code.equals(0)) {
            log.error("头条创意素材获取接口异常==》accountId:{},message:{}", token.getAccountId(), resultObject.getString("message"));
            return;
        }
        JSONArray data = resultObject.getJSONObject("data").getJSONArray("list");
        if (null == data || data.isEmpty()) {
            return;
        }
        List<ByteDanceCreative> creatives = new ArrayList<>();
        for (int i = 0; i < data.size(); i++) {
            JSONObject dataObject = data.getJSONObject(i);
            ByteDanceCreative creative = new ByteDanceCreative(dataObject, token);
            creatives.add(creative);
        }
        creativeMapper.replaceBatch(creatives);
        getVideoByPage(token, date, pageNum + 1);
    }


    public JSONObject getCreative(CtopOauthToken token, String date, int pageNumber) {
        // 请求地址
        String url = "https://ad.oceanengine.com/open_api/2/creative/get/";
        // 请求参数
        JSONObject param = new JSONObject();
        param.put("advertiser_id", token.getAccountId());
        param.put("page", pageNumber);
        param.put("page_size", 100);
        if (null != date && !"".equals(date.trim())) {
            JSONObject filtering = new JSONObject();
            filtering.put("creative_modify_time", date);
            param.put("filtering", filtering);
        }
        return HttpUtils.bytedanceGetRequest(token.getAccessToken(), url, param);
    }

    @Override
    public Map<String, Object> advertiserPlanUpdateStatus(CtopOauthToken token, String adIds, String optStatus) {
        Map<String, Object> resultMap = new HashMap<>();
        JSONArray ids = new JSONArray();
        String[] getIds = adIds.split(StringUtils.COMMA);
        if (getIds.length > 0) {
            for (String getId : getIds) {
                ids.add(Long.parseLong(getId));
            }
        }
        //2: 根据token以及用户id获取用户信息数据
        String url = bytedanceApiUrl + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_ad_update_status");
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", token.getAccessToken());

        JSONObject params = new JSONObject();
        params.put("advertiser_id", token.getAccountId());
        params.put("ad_ids", ids.toJSONString());
        params.put("opt_status", optStatus);
        String result = HttpUtils.httpPostRequest(url, params, headers);
        JSONObject jsonObject = JSONObject.parseObject(result);
        int code = jsonObject.getInteger("code");
        if (code != 0) {
            log.error("广告计划更新状态接口异常==》accountId:{},message:{}", token.getAccountId(), jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "广告计划更新状态接口异常");
            return resultMap;
        }
        JSONObject data = jsonObject.getJSONObject("data");
        if (null == data) {
            log.info("广告计划更新状态异常==》accountId:{},message:{}", token.getAccountId(), jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "广告计划更新状态异常");
            return resultMap;
        }
        JSONArray getAdIds = data.getJSONArray("ad_ids");
        if (null != getAdIds && !getAdIds.isEmpty()) {
            //获取广告计划信息
            getAdvertiserPlan(token, adIds, null, null);
        }
        resultMap.put("code", 0);
        resultMap.put("message", "广告组状态修改成功");
        return resultMap;
    }

    @Override
    public Map<String, Object> advertiserPlanUpdateBid(CtopOauthToken token, String adIds, String bids) {
        Map<String, Object> resultMap = new HashMap<>();
        String[] getadIds = adIds.split(StringUtils.COMMA);
        String[] getBids = bids.split(StringUtils.COMMA);
        JSONArray data = new JSONArray();
        if (getadIds.length > 0) {
            for (int i = 0; i < getadIds.length; i++) {
                JSONObject object = new JSONObject();
                Long adId = Long.parseLong(getadIds[i]);
                Long bid = Long.parseLong(getBids[i]);
                object.put("ad_id", adId);
                object.put("bid", bid);
                data.add(object);
            }
        }
        //2: 根据token以及用户id获取用户信息数据
        String url = bytedanceApiUrl + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_ad_update_bid");
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", token.getAccessToken());

        JSONObject params = new JSONObject();
        params.put("advertiser_id", token.getAccountId());
        params.put("data", data.toJSONString());
        String result = HttpUtils.httpPostRequest(url, params, headers);
        JSONObject jsonObject = JSONObject.parseObject(result);
        int code = jsonObject.getInteger("code");
        if (code != 0) {
            log.info("修改计划出价接口异常==》accountId:{},message:{}", token.getAccountId(), jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "修改计划出价接口异常");
            return resultMap;
        }
        JSONObject getData = jsonObject.getJSONObject("data");
        if (null == getData) {
            log.info("修改计划出价异常==》accountId:{},message:{}", token.getAccountId(), jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "修改计划出价异常");
            return resultMap;
        }
        JSONArray getAdIds = getData.getJSONArray("ad_ids");
        if (null != getAdIds && !getAdIds.isEmpty()) {
            //获取广告计划信息
            getAdvertiserPlan(token, adIds, null, null);
        }
        resultMap.put("code", 0);
        resultMap.put("message", "修改计划出价成功");
        return resultMap;
    }

    @Override
    public Map<String, Object> advertiserPlanUpdateBudget(CtopOauthToken token, String adIds, String budgets) {
        Map<String, Object> resultMap = new HashMap<>();
        JSONArray data = new JSONArray();
        String[] getadIds = adIds.split(StringUtils.COMMA);
        String[] getBudgets = budgets.split(StringUtils.COMMA);
        if (getadIds.length > 0) {
            for (int i = 0; i < getadIds.length; i++) {
                JSONObject object = new JSONObject();
                Long adId = Long.parseLong(getadIds[i]);
                Long budget = Long.parseLong(getBudgets[i]);
                object.put("ad_id", adId);
                object.put("budget", budget);
                data.add(object);
            }
        }
        //2: 根据token以及用户id获取用户信息数据
        String url = bytedanceApiUrl + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_ad_update_budget");
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", token.getAccessToken());

        JSONObject params = new JSONObject();
        params.put("advertiser_id", token.getAccountId());
        params.put("data", data.toJSONString());
        String result = HttpUtils.httpPostRequest(url, params, headers);
        JSONObject jsonObject = JSONObject.parseObject(result);
        Integer code = jsonObject.getInteger("code");
        if (null == code || !code.equals(0)) {
            log.info("修改计划预算接口异常==》accountId:{},message:{}", token.getAccountId(), jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "修改计划预算接口异常");
            return resultMap;
        }
        JSONObject getData = jsonObject.getJSONObject("data");
        if (null == getData) {
            log.info("修改计划预算异常==》accountId:{},message:{}", token.getAccountId(), jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "修改计划预算异常");
            return resultMap;
        }
        JSONArray getAdIds = getData.getJSONArray("ad_ids");
        if (null != getAdIds && !getAdIds.isEmpty()) {
            //获取广告计划信息
            getAdvertiserPlan(token, adIds, null, null);
        }
        resultMap.put("code", 0);
        resultMap.put("message", "修改计划预算成功");
        return resultMap;
    }


    /**
     * 获取流量包数据接口
     *
     * @param accountId
     * @return
     */
    @Override
    public Map<String, Object> flowPackageGet(String accountId) {
        JSONObject jsonObject = flowPackageGetJsonObject(accountId);
        JSONObject result = new JSONObject();
        Integer code = jsonObject.getInteger("code");
        String message = jsonObject.getString("message");
        result.put("code", code);
        result.put("message", message);
        if (null == code || code != 0) {
            result.put("success", false);
            return result;
        }
        JSONArray packageList = jsonObject.getJSONObject("data").getJSONArray("list");
        if (null != packageList && !packageList.isEmpty()) {
            JSONArray packageArray = new JSONArray();
            for (int i = 0; i < packageList.size(); i++) {
                JSONObject packageObject = packageList.getJSONObject(i);
                packageObject.remove("rit");
                String status = packageObject.getString("status");
                if ("FLOW_PACKAGE_ENABLE".equals(status)) {
                    packageArray.add(packageObject);
                }
            }
            result.put("data", packageArray);
        }
        result.put("success", true);
        return result;
    }

    @Override
    public void getMaterialList(CtopOauthToken token) {
        getVideoByPage(token, "", 1);
        getImageByPage(token, "", 1);
    }

    @Autowired
    private IBytedanceImageInfoService imageInfoService;

    @Override
    public void getImageByPage(CtopOauthToken token, String imageIds, int page) {
        // 请求地址
        String url = "https://ad.oceanengine.com/open_api/2/file/image/get/";
        // 请求参数
        JSONObject param = new JSONObject();
        param.put("advertiser_id", token.getAccountId());
        param.put("page", page);
        param.put("page_size", 100);
        if (null != imageIds && !"".equals(imageIds.trim())) {
            String[] getIds = imageIds.split(StringUtils.COMMA);
            if (getIds.length > 0) {
                JSONObject filter = new JSONObject();
                JSONArray idArray = new JSONArray();
                Collections.addAll(idArray, getIds);
                filter.put("image_ids", idArray);
                param.put("filtering", filter);
            }
        }

        JSONObject resultObject = HttpUtils.bytedanceGetRequest(token.getAccessToken(), url, param);
        if (null == resultObject) {
            log.error("获取图片素材库接口异常==》accountId:{}", token.getAccountId());
            return;
        }
        Integer code = resultObject.getInteger("code");
        if (null == code || !code.equals(0)) {
            log.error("获取图片素材库接口异常==》accountId:{},message:{}", token.getAccountId(), resultObject.getString("message"));
            return;
        }
        JSONArray data = resultObject.getJSONObject("data").getJSONArray("list");
        if (null == data || data.isEmpty()) {
            log.info("获取图片素材完成==》accountId:{},message:{}", token.getAccountId(), resultObject.getString("message"));
            return;
        }
        List<BytedanceImageInfo> videoInfoList = new ArrayList<>();
        for (int i = 0; i < data.size(); i++) {
            JSONObject dataObject = data.getJSONObject(i);
            BytedanceImageInfo imageInfo = new BytedanceImageInfo(dataObject, token);
            videoInfoList.add(imageInfo);
        }
        imageInfoService.replaceBatch(videoInfoList);
        getImageByPage(token, imageIds, page + 1);
    }

    private void getVideoByPage(CtopOauthToken token, String videoIds, int page) {
        // 请求地址
        String url = "https://ad.oceanengine.com/open_api/2/file/video/get/";
        // 请求参数
        JSONObject param = new JSONObject();
        param.put("advertiser_id", token.getAccountId());
        param.put("page", page);
        param.put("page_size", 100);
        if (null != videoIds && !"".equals(videoIds.trim())) {
            String[] getIds = videoIds.split(StringUtils.COMMA);
            if (getIds.length > 0) {
                JSONObject filter = new JSONObject();
                JSONArray idArray = new JSONArray();
                Collections.addAll(idArray, getIds);
                filter.put("video_ids", idArray);
                param.put("filtering", filter);
            }
        }

        JSONObject resultObject = HttpUtils.bytedanceGetRequest(token.getAccessToken(), url, param);
        if (null == resultObject) {
            log.error("获取视频素材库接口异常==》accountId:{}", token.getAccountId());
            return;
        }
        Integer code = resultObject.getInteger("code");
        if (null == code || !code.equals(0)) {
            log.error("获取视频素材库接口异常==》accountId:{},message:{}", token.getAccountId(), resultObject.getString("message"));
            return;
        }
        JSONArray data = resultObject.getJSONObject("data").getJSONArray("list");
        if (null == data || data.isEmpty()) {
            log.info("获取视频素材完成==》accountId:{},message:{}", token.getAccountId(), resultObject.getString("message"));
            return;
        }
        List<ByteDanceVideoInfo> videoInfoList = new ArrayList<>();
        for (int i = 0; i < data.size(); i++) {
            JSONObject dataObject = data.getJSONObject(i);
            ByteDanceVideoInfo videoInfo = new ByteDanceVideoInfo(dataObject, token);
            videoInfoList.add(videoInfo);
        }
        videoInfoService.replaceBatch(videoInfoList);
        getVideoByPage(token, videoIds, page + 1);
    }

    public JSONObject flowPackageGetJsonObject(String accountId) {
        CtopOauthToken token = tokenService.getOauthTokenByAccountId(accountId);
        // 请求地址
        String url = "https://ad.toutiao.com/open_api/2/tools/union/flow_package/get/";
        // 请求参数
        JSONObject params = new JSONObject();
        params.put("advertiser_id", token.getAccountId());
        params.put("page", 1);
        params.put("page_size", 100);
        return HttpUtils.bytedanceGetRequest(token.getAccessToken(), url, params);
    }


    @Override
    public void getBytedancePlanList(String accountId, String accessToken, int pageNum, String createDate, String updateDate) {
        // 请求地址
        String url = bytedanceApiUrl + bytedanceAdGetUrl;
        // 请求参数
        Map<String, Object> filtering = new HashMap<>();
        if (null != createDate && !"".equals(createDate)) {
            filtering.put("ad_create_time", createDate);
        }
        if (null != updateDate && !"".equals(updateDate)) {
            filtering.put("ad_modify_time", updateDate);
        }

        JSONObject param = new JSONObject();
        param.put("advertiser_id", accountId);
        param.put("page", pageNum);
        param.put("page_size", 1000);
        param.put("filtering", filtering);
        JSONObject resultObject = HttpUtils.bytedanceGetRequest(accessToken, url, param);
        Integer code = resultObject.getInteger("code");
        if (null == code || !code.equals(0)) {
            log.error("获取广告计划信息接口异常==》accountId:{},message:{}", accountId, resultObject.getString("message"));
            return;
        }
        JSONArray data = resultObject.getJSONObject("data").getJSONArray("list");
        if (null == data || data.isEmpty()) {
            return;
        }
        for (int i = 0; i < data.size(); i++) {
            JSONObject dataObject = data.getJSONObject(i);
            ByteDanceAdvertisePlan advertisePlan = new ByteDanceAdvertisePlan(dataObject, accountId);
            BigDecimal deepCpabid = dataObject.getBigDecimal("deep_cpabid");
            if (null != deepCpabid) {
                advertisePlan.setDeepCpaBid(deepCpabid);
            }
            //   advertisePlanService.saveOrUpdate(advertisePlan);
        }
        getBytedancePlanList(accountId, accessToken, pageNum + 1, createDate, updateDate);
    }


}
